;	C64 MOUSE DRIVER FOR BASIC 2.0 APPLICATIONS
;	IMPROVED BY KEVIN KRAUSNICK ON 17-FEB-2008

;--------------------------------------------------------------------------------

; *** CONSTANTS ***

IIRQ	= $0314
VIC	= $D000		; VIC REGISTERS
SID	= $D400		; SID REGISTERS
POTX	= SID+$19
POTY	= SID+$1A

ACTIVE	= $117E		; IF ZERO, THEN MOVE SPRITE

XPOS	= VIC+$00	; LOW ORDER X POSITION
YPOS	= VIC+$01	; Y POSITION
XPOSMSB	= VIC+$10	; BIT 0 IS HIGH ORDER X POSITION

;--------------------------------------------------------------------------------

; *** JUMP TABLE AND POLL REGISTERS ***

JMP	INSTALL		; INSTALL OR ACTIVATE INTERRUPT SERVICE ROUTINE (ISR)
JMP	UNINST		; UNINSTALL ISR
ROW	*=*+1		; CHARACTER ROW
COL	*=*+1		; CHARACTER COLUMN

;--------------------------------------------------------------------------------

; *** INSTALL OR ACTIVATE INTERRUPT SERVICE ROUTINE (ISR) ***

INSTALL LDA IIRQ+1
	CMP #>MIRQ
	BEQ M99
	PHP
	SEI
	LDA IIRQ
	STA IIRQ2
	LDA IIRQ+1
	STA IIRQ2+1

	LDA #<MIRQ
	STA IIRQ
	LDA #>MIRQ
	STA IIRQ+1
	PLP
M99:	RTS

;--------------------------------------------------------------------------------

; *** UNINSTALL ISR ***

UNINST	LDA IIRQ2+1
	BEQ M8
	PHP
	SEI
	LDA IIRQ2
	STA IIRQ
	LDA IIRQ2+1
	STA IIRQ+1
	LDA #0
	STA IIRQ2+1
	PLP
M8	RTS

;--------------------------------------------------------------------------------

; *** MAIN LOOP OF ISR ***

MIRQ    CLD		; JUST IN CASE.....
	;LDA ACTIVE	; IF    BASIC IS MOVING SPRITE
	;BNE M3		;       LET BASIC HAVE IT ( WHY NOT ? )
	LDA POTX	; GET DELTA VALUES FOR X
	LDY OPOTX
	JSR MOVCHK
	STY OPOTX

	CLC		; MODIFY LOW ORDER X POSITION
	ADC XPOS
	STA XPOS
	STA XPOS+2
	STA XPOS+4
	STA XPOS+6
	STA XPOS+8
	TXA
	ADC #$00
	AND #%00000001
	EOR XPOSMSB
	AND #1
	BEQ M2
	ORA #%11111110
M2	STA XPOSMSB

	LDA POTY	; GET DELTA VALUE FOR Y
	LDY OPOTY
	JSR MOVCHK
	STY OPOTY

	SEC		; MODIFY Y POSITION ( DECREASE Y FOR INCREASE IN POT )
	EOR #$FF
	ADC YPOS
	STA YPOS
	STA YPOS+2
	STA YPOS+4
	STA YPOS+6
	STA YPOS+8

	LDA YPOS	; CONVERT Y PIXEL VALUE TO CHARACTER ROW
	TAY		; ADJUST TO MOUSE POINTER "HOT SPOT"
	DEY
	DEY
	TYA
	LSR		; DIVIDE BY 8
	LSR
	LSR
	TAX
	DEX		; SUBTRACT 6
	DEX
	DEX
	DEX
	DEX
	DEX
	TXA
	CMP #25		; ONLY ALLOW VALID ROW VALUES
	BCC M6
	LDA #24
M6	STA ROW

		
		
	
	LDA XPOSMSB	; CONVERT X PIXEL VALUE TO CHARACTER COLUMN
	AND #1
	STA MTEMP
	LDX XPOS
	LDY #5		; ADJUST TO MOUSE POINTER "HOT SPOT" (5 PIXELS RIGHT)
M6A	INX
	BNE M6B
	INC MTEMP
M6B	DEY
	BNE M6A
	CLC
	LDA MTEMP
	BEQ M6C
	SEC
M6C	TXA
	ROR		; DIVIDE BY 8, INCLUDING MSB
	LSR
	LSR
	TAX
	DEX		; SUBSTRACT 3
	DEX
	DEX
	TXA
	CMP #40		; ONLY ALLOW VALID COLUMN VALUES
	BCC M9
	LDA #39
M9	STA COL

M3	JMP (IIRQ2)	; CONTINUE W/ IRQ OPERATION

;--------------------------------------------------------------------------------

; *** CHECK FOR MOUSE MOVEMENT ***

; MOVCHK
;       ENTRY   Y    = OLD VALUE OF POT REGISTER
;               A    = CURRENT VALUE OF POT REGISTER
;       EXIT    Y    = VALUE TO USE FOR OLD VALUE
;               X, A = DELTA VALUE FOR POSITION

MOVCHK	STY OLDVALUE	; SAVE OLD &  NEW VALUES
	STA NEWVALUE
	LDX #0		; PRELOAD X W/ 0

	SEC		; A <=  MOD64( NEW-OLD )
	SBC OLDVALUE
	AND #%01111111
	CMP #%01000000	; IF      > 0
	BCS M4
	LSR		;         A <= A/2
	BEQ M5		;         IF      <> 0
	LDY NEWVALUE	;                 Y <= NEWVALUE
	RTS		;                 RETURN

M4	ORA #%11000000	; ELSE    OR IN HIGH ORDER BITS
	CMP #$FF	;         IF      <> -1
	BEQ M5
	SEC		;                 A <= A/2
	ROR
	LDX #$FF	;                 X <= -1
	LDY NEWVALUE	;                 Y <= NEWVALUE
	RTS		;                 RETURN

M5	LDA #0		; A <= 0
	RTS		; RETURN W/ Y = OLD VALUE

;--------------------------------------------------------------------------------

; *** INTERNAL PROGRAM VARIABLES ***

IIRQ2		*=*+2
OPOTX		*=*+1
OPOTY		*=*+1
NEWVALUE	*=*+1
OLDVALUE	*=*+1
MTEMP		*=*+1
