; ************************************************
; *** Sound Skulptor for the Commodore 64      ***
; *** 12-Mar-2008 by Kevin Krausnick           ***
; *** Ported to Acme Assembler on 14-Sep-2024  ***
; ************************************************

;	CONCEPT BASED ON ARTICLE IN 5/84 EDITION OF COMPUTE'S GAZETTE MAGAZINE

; Memory map:
; $0801 - $2500 (roughly) code, mouse driver and uninitialized data
; $2800 - $2940 mouse pointer sprites
; $3000 - $3100 sounds
; $3800 - $4000 custom character set

CHROUT  = $FFD2
GETIN   = $FFE4
SCREENRAM = $0400
PLOT    = $FFF0

* = $801		; 2049 - START ML PROGRAM FROM BASIC

; *** BASIC LOADER, CREDITS AND INSTRUCTIONS ***

!BYTE 40,8,10,0,143,32,83,79,85,78,68,32,83,75,85,76
!BYTE 80,84,79,82,32,45,32,75,69,86,73,78,32,75,82,65
!BYTE 85,83,78,73,67,75,0,73,8,20,0,158,32,50,51,50
!BYTE 49,58,143,32,75,69,89,66,79,65,82,68,32,83,72,79
!BYTE 82,84,67,85,84,83,58,0,104,8,30,0,143,32,49,45
!BYTE 51,32,32,84,79,71,71,76,69,32,86,79,73,67,69,32
!BYTE 79,78,47,79,70,70,0,140,8,40,0,143,32,52,32,32
!BYTE 32,32,84,79,71,71,76,69,32,65,76,76,32,86,79,73
!BYTE 67,69,83,32,79,78,47,79,70,70,0,178,8,50,0,143
!BYTE 32,84,32,32,32,32,84,82,73,71,71,69,82,83,32,83
!BYTE 79,85,78,68,32,77,79,77,69,78,84,65,82,73,76,89
!BYTE 0,204,8,60,0,143,32,70,51,32,32,32,80,82,69,86
!BYTE 73,79,85,83,32,83,79,85,78,68,0,226,8,70,0,143
!BYTE 32,70,53,32,32,32,78,69,88,84,32,83,79,85,78,68
!BYTE 0,247,8,80,0,143,32,70,55,32,32,32,78,69,88,84
!BYTE 32,77,69,78,85,0,15,9,90,0,143,32,83,84,79,80
!BYTE 32,69,88,73,84,32,80,82,79,71,82,65,77,0,0,0

;--------------------------------------------------------------------------------

; *** MAIN PROGRAM LOOP ***

	JSR INITIALIZE	; INITIALIZATION ROUTINE
	JSR CUSTCHAR	; SET UP CUSTOM CHARACTERS
	JSR INSTALL	; INSTALL OR ACTIVATE MOUSE DRIVER ISR
	JSR SHOWPTR	; SHOW MOUSE POINTER
	JSR UPDATESCR	; SHOW DEFAULT MENU (V1) UPON PROGRAM STARTUP
M1:	JSR POLLMOUSE	; CHECK FOR AND PROCESS MOUSE CLICKS
	JSR HOTSPOT	; REVEAL MOUSE POINTER HOT SPOTS
;	JSR SHOWMOUSPOS	; SHOW MOUSE POSITION (FOR DEBUGGING)
	JSR POLLJOY	; POLL FOR JOYSTICK MOVEMENT
	JSR POLLKEYBD	; CHECK FOR AND PROCESS KEYSTROKES
	LDA EXITFLAG
	BEQ M1
	LDA BUTTON	; CHECK LEFT MOUSE BUTTON
	AND #%00010000  ; WAIT UNTIL MOUSE BUTTON IS RELEASED,
	BEQ M1		;  SO MENU SELECTION CAN BE SEEN
EXIT	JSR CLEANUP	; PREPARE TO EXIT PROGRAM
	RTS		; END PROGRAM

;--------------------------------------------------------------------------------

; *** CONSTANTS ***

CLS		= 147	; CLEAR SCREEN
RIGHT		= 29	; CURSOR RIGHT
SPACE		= 32	; SPACE
QUOTE		= 34	; QUOTATION MARK
RETURN		= 13	; CARRAIGE RETURN
BACKSPACE	= 20	; BACKSPACE
REVERSEON	= 18	; REVERSE ON
REVERSEOFF	= 146	; REVERSE OFF
UNCHECKED	= 214	; CBM ASCII VALUE OF CHECK BOX (OFF)
CHECKED		= 190	; CBM ASCII VALUE OF CHECKED BOX (ON)
SLIDER		= 218	; CBM ASCII VALUE OF SLIDER HORIZONTAL CONTROL
RADIOOFF	= 209	; CBM ASCII VALUE OF RADIO BUTTON UNSELECTED
RADIOON		= 215	; CBM ASCII VALUE OF RADIO BUTTON SELECTED
F3		= 5	; KEYSCAN VALUE FOR F3 KEY - DECREASES SELECTED SOUND #
F5		= 6	; KEYSCAN VALUE FOR F5 KEY - INCREASES SELECTED SOUND #
F7		= 3	; KEYSCAN VALUE FOR F7 KEY - CYCLES BETWEEN MENUS
KEY1		= 56	; KEYSCAN VALUE FOR 1 KEY - TOGGLES VOICE 1
KEY2		= 59	; KEYSCAN VALUE FOR 2 KEY - TOGGLES VOICE 2
KEY3		= 8	; KEYSCAN VALUE FOR 3 KEY - TOGGLES VOICE 3
KEY4		= 11	; KEYSCAN VALUE FOR 4 KEY - TOGGLES ALL VOICES
KEYT		= 22	; KEYSCAN VALUE FOR T KEY - TRIGGERS ALL VOICES FOR CURRENT SOUND
RUNSTOP		= 63	; KEYSCAN VALUE FOR RUN/STOP KEY - EXITS PROGRAM
BUTTON		= 56321	; MOUSE BUTTON CLICK STATUS (PORT 1)
FIRE		= 56320	; JOYSTICK FIRE BUTTON STATUS (PORT 2)
JOYPORT2	= 56320	; JOYSTICK PORT 2
BORDER		= 53280	; BORDER COLOR
BACKGROUND	= 53281	; BACKGROUND COLOR
CURSORCOLOR	= 646	; CURSOR COLOR
DARKGRAY	= 12	; COLOR DARK GRAY
LIGHTGRAY	= 15	; COLOR LIGHT GRAY
BLACK		= 0	; COLOR BLACK
TIME		= 162	; JIFFY CLOCK TIME REGISTER
KEYCOUNT	= 198	; # CHARACTERS IN KEYBOARD BUFFER
BLNSW		= 204	; CURSOR BLINK ENABLE: 0 = FLASH CURSOR
BLON		= 207	; FLAG: WAS LAST CURSOR BLINK ON OR OFF
KEYQUEUE	= 631	; KEYBOARD BUFFER QUEUE
KEYMATRIX	= 203	; KEYBOARD SCAN MATRIX
SHFLAG		= 653	; SHIFT/CTRL/CMD KEYPRESS MATRIX
SOUNDTABLE	= $3000	; SID SOUNDS (25 REGISTERS * 10 = 250 BYTES)
SOUNDNO		= SOUNDTABLE + 250	; SOUND # (0-9)
SOUNDTEND	= SOUNDNO + 1		; END OF SOUNDS TABLE
STAMP		= SOUNDTEND + 1		; DETERMINES WHETHER TO INITIALIZE SOUNDS

;--------------------------------------------------------------------------------

; *** PROGRAM VARIABLES AND STRINGS ***

MENUNO		!BYTE 0	; 0-2 = VOICE 1-3, 3 = FILTER / MASTER VOLUME, 4 = DONE
SOUNDFLAG	!BYTE 0	; FLAG TO CLEAR SELECTED SOUND OR ALL SOUNDS
VOFFSET		!BYTE 0	; OFFSET FROM BASE OF SID REGISTERS FOR SELECTED VOICE
IGNOREMOUSE	!BYTE 0	; FLAG TO KEEP TRACK OF MOUSE BUTTON PRESS
IGNOREKEYBD	!BYTE 0	; FLAG TO KEEP TRACK OF KEYPRESSES
LASTBUTTONY	!BYTE 0 ; TRACK CHANGES TO MOUSE POSITION (Y) WHILE BUTTON IS PRESSED
LASTBUTTONX	!BYTE 0 ; TRACK CHANGES TO MOUSE POSITION (X) WHILE BUTTON IS PRESSED
ALLOWSLIDE	!BYTE 0 ; FLAG TO PREVENT/ALLOW MOUSE BUTTON EVENTS WHILE SLIDING
SPINFLAG	!BYTE 0 ; FLAG TO ALLOW MULTIPLE BUTTON EVENTS WHILE PRESSING SPINNER
SPINDELAY	!BYTE 0 ; DELAY BEFORE SPINNER BUTTON EVENT REPEATS
SPINREPEAT	!BYTE 0 ; DELAY BETWEEN REPEATED SPINNER BUTTON EVENTS
CURSORY		!BYTE 0	; CURSOR ROW NUMBER TO MOVE TO
CURSORX		!BYTE 0	; CURSOR COLUMN NUMBER TO MOVE TO
ALLVOICES	!BYTE 0 ; FLAG TO SWITCH ALL VOICES ON/OFF
EXITFLAG	!BYTE 0	; FLAG TO KEEP TRACK OF CONDITIONS FOR EXITING PROGRAM
LASTSLIDEH	!BYTE 0	; PREVENT REDUNDANT SCREEN REDRAWS OF FREQ HI SLIDER
LASTSLIDEL	!BYTE 0	; PREVENT REDUNDANT SCREEN REDRAWS OF FREQ LO SLIDER
LASTSLIDEFH	!BYTE 0	; PREVENT REDUNDANT SCREEN REDRAWS OF FILTER FREQ HI SLIDER
OLDVMCSB	*=*+1	; SAVE VALUE FOR VIC-II MEMORY CONTROL REGISTER (53272)
DECVAL		*=*+2	; DECIMAL VALUE OF A BYTE (FOR ON-SCREEN DISPLAY)
OLDBORDER	*=*+1	; SAVE ORIGINAL BORDER COLOR
OLDBKGRND	*=*+1	; SAVE ORIGINAL BACKGROUND COLOR
OLDCCOLOR	*=*+1	; SAVE ORIGINAL CURSOR COLOR
TEMP		*=*+1	; GENERAL PURPOSE SCRATCH VARIABLE

FILELEN		!BYTE 0	; LENGTH OF FILENAME
FILENAME = *		; NAME OF FILE TO BE LOADED OR SAVED
!TEXT "                "
!BYTE 0

MSG = *
!BYTE 4,1,86,49,0	; "V1"
!BYTE 6,1,86,50,0	; "V2"
!BYTE 8,1,86,51,0	; "V3"
!BYTE 10,1,70,73,76,84,0	; "FILT"
!BYTE 12,1,REVERSEON
!TEXT "DONE"
!BYTE REVERSEOFF,0

;--------------------------------------------------------------------------------

; *** SID REGISTER SHADOW VARIABLES ***

SIDDEFAULTS = *		; USE VARIABLES TO KEEP TRACK OF WRITE-ONLY SID REGISTERS

!BYTE 0			; R00: VOICE 1 FREQUENCY - LOW BYTE
!BYTE 0			; R01: VOICE 1 FREQUENCY - HIGH BYTE
!BYTE 0			; R02: VOICE 1 PULSE WIDTH - LOW BYTE
!BYTE %00000111		; R03: VOICE 1 PULSE WIDTH - HIGH NYBBLE
!BYTE 0			; R04: VOICE 1 WAVEFORM, SYNC, RING, TEST & GATE BITS
!BYTE 0			; R05: VOICE 1 ATTACK / DECAY
!BYTE 0			; R06: VOICE 1 SUSTAIN / RELEASE
!BYTE 0			; R07: VOICE 2 FREQUENCY - LOW BYTE
!BYTE 0			; R08: VOICE 2 FREQUENCY - HIGH BYTE
!BYTE 0			; R09: VOICE 2 PULSE WIDTH - LOW BYTE
!BYTE %00000111		; R10: VOICE 2 PULSE WIDTH - HIGH NYBBLE
!BYTE 0			; R11: VOICE 2 WAVEFORM, SYNC, RING, TEST & GATE BITS
!BYTE 0			; R12: VOICE 2 ATTACK / DECAY
!BYTE 0			; R13: VOICE 2 SUSTAIN / RELEASE
!BYTE 0			; R14: VOICE 3 FREQUENCY - LOW BYTE
!BYTE 0			; R15: VOICE 3 FREQUENCY - HIGH BYTE
!BYTE 0			; R16: VOICE 3 PULSE WIDTH - LOW BYTE
!BYTE %00000111		; R17: VOICE 3 PULSE WIDTH - HIGH NYBBLE
!BYTE 0			; R18: VOICE 3 WAVEFORM, SYNC, RING, TEST & GATE BITS
!BYTE 0			; R19: VOICE 3 ATTACK / DECAY
!BYTE 0			; R20: VOICE 3 SUSTAIN / RELEASE
!BYTE 0			; R21: LOW PORTION OF FILTER CUT-OFF FREQUENCY
!BYTE 0			; R22: FILTER CUT-OFF FREQUENCY - HIGH BYTE
!BYTE 0			; R23: FILTER VOICE SELECT & RESONANCE CONTROL
!BYTE %00000111		; R24: FILTER TYPE SELECT AND MASTER VOLUME

SIDREGS 	*=*+25	; SID VALUES IN USE

;--------------------------------------------------------------------------------

; *** INITIALIZATION ROUTINE ***

INITIALIZE = *

	LDA #0
	STA EXITFLAG
	STA MENUNO	; MENU # FOR VOICE 1
	STA VOFFSET	; OFFSET FROM BASE OF SID REGISTERS FOR SELECTED VOICE

	LDA BORDER
	STA OLDBORDER
	LDA #DARKGRAY
	STA BORDER
	LDA BACKGROUND
	STA OLDBKGRND
	LDA #LIGHTGRAY
	STA BACKGROUND
	LDA CURSORCOLOR
	STA OLDCCOLOR
	LDA #BLACK
	STA CURSORCOLOR
	LDA #CLS	; CLEAR SCREEN
	JSR CHROUT
	LDA #0
	STA SOUNDNO
	LDA #'K'
	CMP STAMP
	BEQ INIT1
	STA STAMP
	JSR CLEARALL	; SET SID REGISTERS TO DEFAULT VALUES FOR ALL SOUNDS
INIT1	JSR UPDATESID
	LDA #0
	RTS

;--------------------------------------------------------------------------------

; *** CLEAR ALL SOUNDS ***

CLEARALL = *

	LDA SOUNDNO
	PHA
	LDX #0
CA1	STX SOUNDNO
	JSR CLEARSOUND
	INX
	CPX #10
	BNE CA1
	PLA
	STA SOUNDNO
	JSR INITSID	; INITIALIZE SID REGISTERS
	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH OUR SHADOW REGISTERS
	RTS

; *** CLEAR SELECTED SOUND ***

CLEARSOUND = *

	TXA
	PHA
	LDX #0
CLRS1	LDA SIDDEFAULTS, X
	STA SIDREGS, X
	INX
	CPX #25
	BNE CLRS1
	JSR SID2SOUND
	PLA
	TAX
	RTS

; *** INITIALIZE SID REGISTERS ***

INITSID = *

	LDX #0
	TXA
IS1	STA SID, X
	INX
	CPX #25
	BNE IS1
	RTS

; *** UPDATE SID REGISTERS TO MATCH OUR SHADOW REGISTERS ***

UPDATESID = *

	TXA
	PHA
	LDX #0

USR1	CPX #4
	BNE USR2
	JSR GATEOFF
	JMP USR5

USR2	CPX #4 + 7
	BNE USR3
	JSR GATEOFF
	JMP USR5

USR3	CPX #4 + 14
	BNE USR4
	JSR GATEOFF
	JMP USR5

USR4	LDA SIDREGS, X
	STA SID, X

USR5	INX
	CPX #25
	BNE USR1
	JSR GATESON
	JSR SID2SOUND
	PLA
	TAX
	RTS

GATEOFF = *

	LDA SIDREGS, X
	AND #%11111110
	STA SID, X
	RTS

GATESON = *

	LDA SIDREGS + 4	; UPDATE GATE REGISTERS LAST
	STA SID + 4
	LDA SIDREGS + 4 + 7
	STA SID + 4 + 7
	LDA SIDREGS + 4 + 14
	STA SID + 4 + 14
	RTS

SID2SOUND = *

	PHA
	TXA
	PHA
	TYA
	PHA
	LDA #0
	LDY SOUNDNO
	BEQ S2S2
	CLC
S2S1	ADC #25
	DEY
	BNE S2S1
	TAY
S2S2	LDX #0
S2S3	LDA SIDREGS, X
	STA SOUNDTABLE, Y
	INY
	INX
	CPX #25
	BNE S2S3
	PLA
	TAY
	PLA
	TAX
	PLA
	RTS

SOUND2SID = *

	PHA
	TXA
	PHA
	TYA
	PHA
	LDA #0
	LDY SOUNDNO
	BEQ S2S5
	CLC
S2S4	ADC #25
	DEY
	BNE S2S4
	TAY
S2S5	LDX #0
S2S6	LDA SOUNDTABLE, Y
	STA SIDREGS, X
	INY
	INX
	CPX #25
	BNE S2S6
	PLA
	TAY
	PLA
	TAX
	PLA
	RTS

;--------------------------------------------------------------------------------

; *** FUNCTIONS THAT TOGGLE OR SWITCH VOICES ON OR OFF ***

TOGGLEV1 = *		; TOGGLE VOICE 1

	LDX #0
	JSR TOGGLEVOICE
	JSR V1BOX
	RTS

TOGGLEV2 = *		; TOGGLE VOICE 2

	LDX #7
	JSR TOGGLEVOICE
	JSR V2BOX
	RTS

TOGGLEV3 = *		; TOGGLE VOICE 3

	LDX #14
	JSR TOGGLEVOICE
	JSR V3BOX
	RTS

TOGGLEVOICE = *		; TOGGLE VOICE ON OR OFF

	LDA #%00000001
	EOR SIDREGS + 4, X
	STA SIDREGS + 4, X
	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH SHADOW REGISTERS
	RTS

VOICEON = *		; SWITCH VOICE ON

	LDA SIDREGS + 4, X
	ORA #%00000001
	STA SIDREGS + 4, X
	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH SHADOW REGISTERS
	RTS

VOICEOFF = *		; SWITCH VOICE OFF

	LDA SIDREGS + 4, X
	AND #%11111110
	STA SIDREGS + 4, X
	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH SHADOW REGISTERS
	RTS

;--------------------------------------------------------------------------------

; *** PREPARE TO EXIT PROGRAM ***

CLEANUP = *

	JSR INITSID	; INITIALIZE SID REGISTERS (TURN-OFF SOUND)
	JSR HIDEPTR	; HIDE MOUSE POINTER
	JSR UNINST	; UNINSTALL MOUSE DRIVER ISR
	LDA OLDBORDER
	STA BORDER
	LDA OLDBKGRND
	STA BACKGROUND
	LDA OLDCCOLOR
	STA CURSORCOLOR
	LDA #CLS	; CLEAR SCREEN
	JSR CHROUT
	LDA #0
	STA KEYCOUNT	; CLEAR KEYBOARD BUFFER
	LDA OLDVMCSB
	STA 53272	; RESTORE ORIGINAL SYSTEM FONT
	RTS


;--------------------------------------------------------------------------------

; *** POLL FOR JOYSTICK MOVEMENT ***

POLLJOY = *

	LDA #0
	STA JDFLAG
	LDA JOYPORT2
	LSR
	BCS PJ1
	JSR JOYUP
PJ1	LSR
	BCS PJ2
	JSR JOYDOWN
PJ2	LSR
	BCS PJ3
	JSR JOYLEFT
PJ3	LSR
	BCS PJ4
	JSR JOYRIGHT
PJ4	RTS

JOYUP = *

	DEC YPOS
	JSR JOYDELAY
	RTS

JOYDOWN = *

	INC YPOS
	JSR JOYDELAY
	RTS

JOYLEFT = *

	PHA
	DEC XPOS
	LDA XPOS
	CMP #255
	BNE JL1

	LDA XPOSMSB
	AND #%00000111
	EOR #%00000111
	STA XPOSMSB
	BEQ JL1
	LDA #100
	STA XPOS

JL1	JSR JOYDELAY
	PLA
	RTS

JOYRIGHT = *

	PHA
	INC XPOS
	BNE JR1

JR1A	LDA XPOSMSB
	AND #%00000111
	EOR #%00000111
	STA XPOSMSB
	JMP JREND

JR1	LDA XPOS
	CMP #101
	BCC JREND
	LDA XPOSMSB
	AND #%00000001
	BEQ JREND
	LDA #0
	STA XPOS
	JMP JR1A

JREND	JSR JOYDELAY
	PLA
	RTS

JOYDELAY = *

	PHA
	LDA JDFLAG
	BNE JDEND
	LDY #8
	LDX #0
JD1	DEX
	BNE JD1
	DEY
	BNE JD1
	LDA #1
	STA JDFLAG
JDEND	PLA
	RTS

JDFLAG	*=*+1		; PREVENT MULTIPLE DELAYS WHEN MOVING DIAGONALLY

;--------------------------------------------------------------------------------

; *** CHECK FOR AND PROCESS KEYSTROKES ***

POLLKEYBD = *

; MOUSE CLICKS SOMETIMES GENERATE FAUX KEYBOARD EVENTS, 
; SO SKIP KEYBOARD CHECK ROUTINE IF THE MOUSE BUTTON IS PRESSED

	LDA BUTTON
	AND #%00010000	; CHECK LEFT MOUSE BUTTON
	BNE PK0		; LEFT MOUSE BUTTON PRESSED - SO SKIP ROUTINE
	LDA #0
	STA KEYCOUNT
	JMP PKEND

PK0	JSR GETIN	; CHECK WHETHER KEY HAS BEEN PRESSED - BUT DON'T RELY ON VALUE
	BNE PK1
	STA IGNOREKEYBD
	JMP PKEND

PK1	LDA IGNOREKEYBD
	BEQ PK1A
	JMP PKEND

PK1A	LDA SHFLAG	; ONLY ACCEPT KEYPRESS WITH NO SHIFT, CTRL OR CMD MODIFIER
	BEQ PK2
	JMP PKEND

PK2	LDA KEYMATRIX	; HELPS AVOID FAUX KEYPRESSES CAUSED BY THE MOUSE
	CMP #RUNSTOP	; RUN/STOP EXITS PROGRAM
	BNE PK3
	LDA #1
	STA EXITFLAG
	JMP PKEND

PK3	CMP #F7		; F7 KEY CYCLES BETWEEN MENU SCREENS
	BNE PK4
PK3B	INC MENUNO
	LDA MENUNO
	CMP #5
	BNE PK3A
	LDA #0
	STA MENUNO
PK3A	JSR UPDATESCR
	JMP PKEND

PK4	CMP #F3		; F3 KEY DECREASES SELECTED SOUND #
	BNE PK9
	LDA SOUNDNO
	BEQ PK4A
	DEC SOUNDNO
	JSR SOUND2SID
	JSR UPDATESID
	JSR UPDATESCR
PK4A	JMP PKEND

PK9	CMP #F5		; F5 KEY - INCREASES SELECTED SOUND #
	BNE PK10
	LDA SOUNDNO
	CMP #9
	BCS PK9A
	INC SOUNDNO
	JSR SOUND2SID
	JSR UPDATESID
	JSR UPDATESCR
PK9A	JMP PKEND

PK10	CMP #KEY1	; 1 TOGGLES VOICE 1
	BNE PK5
	JSR TOGGLEV1
	JMP PKEND

PK5	CMP #KEY2	; 2 TOGGLES VOICE 2
	BNE PK6
	JSR TOGGLEV2
	JMP PKEND

PK6	CMP #KEY3	; 3 TOGGLES VOICE 3
	BNE PK7
	JSR TOGGLEV3
	JMP PKEND

PK7	CMP #KEY4	; TOGGLE ALL VOICES ON OR OFF
	BNE PK8
	LDA #%00000001
	EOR ALLVOICES
	STA ALLVOICES
	BNE PK7B
	LDX #0
	JSR VOICEOFF
	LDX #7
	JSR VOICEOFF
	LDX #14
	JSR VOICEOFF
PK7A	JSR V1BOX
	JSR V2BOX
	JSR V3BOX
	JMP PKEND
PK7B	LDX #0
	JSR VOICEON
	LDX #7
	JSR VOICEON
	LDX #14
	JSR VOICEON
	JMP PK7A

PK8	CMP #KEYT	; T TRIGGERS ALL VOICES FOR CURRENT SOUND
	BNE PK11
	LDA #0
	STA ALLVOICES
	LDX #0
	JSR VOICEOFF
	JSR VOICEON
	JSR VOICEOFF
	LDX #7
	JSR VOICEOFF
	JSR VOICEON
	JSR VOICEOFF
	LDX #14
	JSR VOICEOFF
	JSR VOICEON
	JSR VOICEOFF
	JMP PK7A

PK11=*

PKEND	RTS

;--------------------------------------------------------------------------------

; *** CHECK FOR AND PROCESS MOUSE CLICKS ***

; BIT 4 REPRESENTS LEFT MOUSE BUTTON (LOW WHEN CLICKED)
; BIT 0 REPRESENTS RIGHT MOUSE BUTTON (LOW WHEN CLICKED)

POLLMOUSE = *

	LDA BUTTON
	AND FIRE
	AND #%00010000	; CHECK LEFT MOUSE BUTTON
	BEQ PM1		; LEFT MOUSE BUTTON PRESSED

	LDA #0		; NO BUTTON PRESSED,
	STA IGNOREMOUSE	;  SO RESET RELEVANT FLAGS AND EXIT SUBROUTINE
	STA SPINFLAG
	STA SPINDELAY
	STA SPINREPEAT
	STA ALLOWSLIDE
	JMP PMEND

PM1	LDA #0
	STA KEYCOUNT	; REMOVE ANY FAUX KEYBOARD EVENTS
	STA GETIN

	LDA IGNOREMOUSE
	BEQ PM2		; ACKNOWLEDGE BUTTON PRESS
	LDA ALLOWSLIDE
	BEQ PM5		; A SLIDER CONTROL WAS NOT CLICKED

	LDA ROW		; A SLIDER CONTROL WAS CLICKED
	CMP LASTBUTTONY
	BEQ PM1A
	JMP PMEND	; MOUSE Y POSITION HAS CHANGED, SO PREVENT SLIDE
PM1A	LDA COL
	CMP LASTBUTTONX
	BNE PM2		; MOUSE Y POSITION HAS CHANGED, SO ALLOW NEW SLIDE VALUE
	JMP PMEND

PM5	LDA SPINFLAG
	BEQ PMEND	; A SPIN CONTROL WAS NOT CLICKED

	LDA SPINDELAY	; A SPIN CONTROL WAS CLICKED
	CMP #30		; 30 JIFFIES = 1/2 SECOND INITIAL DELAY
	BCS PM6A	; INITIAL DELAY PERIOD HAS ELAPSED
	INC SPINDELAY
	JSR DELAY
	JMP PMEND	; DELAY PERIOD HAS NOT YET ELAPSED, SO CONTINUE WAITING
PM6A	LDA SPINREPEAT
	CMP #2
	BCC PM6B
	LDA #0
	STA SPINREPEAT
	JMP PM2		; PROCESS CLICK EVENT
PM6B	INC SPINREPEAT
	JSR DELAY
	JMP PMEND	; DELAY PERIOD HAS NOT YET ELAPSED, SO CONTINUE WAITING

PM2	LDA #1
	STA IGNOREMOUSE	; PREVENT REPEAT PROCESSING OF CLICK UNTIL BUTTON IS RELEASED
	LDA ROW
	STA LASTBUTTONY
	LDA COL
	STA LASTBUTTONX

	LDA COL
	CMP #5
	BCS PM3
	JSR MENUCLICK	; USER CLICKED SOMEWHERE IN THE MENU AREA
	JMP PM7
PM3	LDA MENUNO
	CMP #3
	BCS PM3A
	JSR VOICECLICK	; USER CLICKED SOMEWHERE ON THE VOICE SCREEN
	JMP PM7
PM3A	CMP #4
	BNE PM4
	JSR DONECLICK	; USER CLICKED SOMEWHERE ON THE DONE SCREEN
	JMP PM7
PM4	JSR FILTERCLICK	; USER CLICKED SOMEWHERE ON THE FILTER SCREEN

PM7	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH OUR SHADOW REGISTERS

PMEND	RTS

DELAY	PHA
	LDA TIME
DLY1	CMP TIME
	BEQ DLY1
	PLA
	RTS

;--------------------------------------------------------------------------------

; *** REVEAL MOUSE POINTER HOT SPOTS BASED ON CURSOR POSITION ***

HOTSPOT = *

	LDA #0
	TAX
	TAY
	STA HOT

	LDA MENUNO
	CMP #3
	BCS HS4
	LDA #<HOTVOICE
	STA $FC
	LDA #>HOTVOICE
	STA $FD
	JMP HS1

HS4	CMP #3
	BNE HS5
	LDA #<HOTFILT
	STA $FC
	LDA #>HOTFILT
	STA $FD
	JMP HS1

HS5	LDA #<HOTDONE
	STA $FC
	LDA #>HOTDONE
	STA $FD

HS1	LDA ($FC), Y
	BEQ HSDONE
	CMP ROW
	BNE HS2
	INY
	LDA ($FC), Y
	CMP COL
	BCS HS2
	INY
	LDA ($FC), Y
	CMP COL
	BCC HS2
	LDA #1
	STA HOT
	JMP HSDONE
HS2	INX
	INX
	INX
	TXA
	TAY
	JMP HS1

HSDONE	LDA HOT
	BEQ HS3
	LDA #%00011000
	CMP 53269
	BEQ HSEND
	STA 53269
	JMP HSEND

HS3	LDA #%00000111
	CMP 53269
	BEQ HSEND
	STA 53269

HSEND	RTS

HOT *=*+1		; HOT SPOT FLAG

HOTVOICE = *
!BYTE 4,0,2
!BYTE 4,3,4
!BYTE 6,0,2
!BYTE 6,3,4
!BYTE 8,0,2
!BYTE 8,3,4
!BYTE 10,0,4
!BYTE 12,0,4
!BYTE 16,0,1
!BYTE 16,3,4
!BYTE 3,13,14
!BYTE 3,17,18
!BYTE 3,19,35
!BYTE 4,13,14
!BYTE 4,17,18
!BYTE 4,19,35
!BYTE 8,19,35
!BYTE 9,19,35
!BYTE 10,19,35
!BYTE 11,19,35
!BYTE 14,19,36
!BYTE 16,19,36
!BYTE 22,19,35
!BYTE 16,8,16
!BYTE 18,8,16
!BYTE 20,8,16
!BYTE 22,8,16
!BYTE 0,0,0

HOTFILT = *
!BYTE 4,0,2
!BYTE 4,3,4
!BYTE 6,0,2
!BYTE 6,3,4
!BYTE 8,0,2
!BYTE 8,3,4
!BYTE 10,0,4
!BYTE 12,0,4
!BYTE 16,0,1
!BYTE 16,3,4
!BYTE 4,13,14
!BYTE 4,17,18
!BYTE 4,19,35
!BYTE 5,13,14
!BYTE 5,17,18
!BYTE 5,19,35
!BYTE 10,8,15
!BYTE 12,8,15
!BYTE 14,8,15
!BYTE 10,18,21
!BYTE 10,23,26
!BYTE 10,28,31
!BYTE 10,33,36
!BYTE 14,19,35
!BYTE 19,12,28
!BYTE 22,11,28
!BYTE 0,0,0

HOTDONE = *
!BYTE 4,0,2
!BYTE 4,3,4
!BYTE 6,0,2
!BYTE 6,3,4
!BYTE 8,0,2
!BYTE 8,3,4
!BYTE 10,0,4
!BYTE 12,0,4
!BYTE 16,0,1
!BYTE 16,3,4
!BYTE 4,8,35
!BYTE 8,8,13
!BYTE 8,17,22
!BYTE 12,8,13
!BYTE 12,16,23
!BYTE 12,25,30
!BYTE 16,8,13
!BYTE 0,0,0

;--------------------------------------------------------------------------------

; *** USER CLICKED SOMEWHERE IN THE MENU AREA ***

MENUCLICK = *

	LDA ROW
	CMP #4
	BNE MC3
	LDA COL
	CMP #4
	BNE MC1

	JSR TOGGLEV1	; TOGGLE VOICE 1 ON/OFF
	JMP MCEND

MC1	CMP #0
	BEQ MC3
	CMP #3
	BCS MC3
	LDA #0		; VOICE 1 MENU SELECTED
	CMP MENUNO
	BEQ MC2
	STA MENUNO
	JSR UPDATESCR
MC2	JMP MCEND

MC3	CMP #6
	BNE MC6
	LDA COL
	CMP #4
	BNE MC4

	JSR TOGGLEV2	; TOGGLE VOICE 2 ON/OFF
	JMP MCEND

MC4	CMP #0
	BEQ MC6
	CMP #3
	BCS MC6
	LDA #1		; VOICE 2 MENU SELECTED
	CMP MENUNO
	BEQ MC5
	STA MENUNO
	JSR UPDATESCR
MC5	JMP MCEND

MC6	CMP #8
	BNE MC9
	LDA COL
	CMP #4
	BNE MC7

	JSR TOGGLEV3	; TOGGLE VOICE 3 ON/OFF
	JMP MCEND

MC7	CMP #0
	BEQ MC9
	CMP #3
	BCS MC9
	LDA #2		; VOICE 3 MENU SELECTED
	CMP MENUNO
	BEQ MC8
	STA MENUNO
	JSR UPDATESCR
MC8	JMP MCEND

MC9	CMP #10
	BNE MC11
	LDA COL
	CMP #0
	BEQ MC11
	CMP #5
	BCS MC11
	LDA #3		; FILTER MENU SELECTED
	CMP MENUNO
	BEQ MC10
	STA MENUNO
	JSR UPDATESCR
MC10	JMP MCEND

MC11	CMP #12		; DONE MENU SELECTED
	BNE MC13
	LDA COL
	CMP #0
	BEQ MC12
	CMP #5
	BCS MC12
	LDA #4
	CMP MENUNO
	BEQ MC12
	STA MENUNO
	JSR UPDATESCR
MC12	JMP MCEND

MC13	CMP #16
	BNE MC14
	LDA COL
	CMP #1
	BEQ MC13A
	CMP #4
	BEQ MC13B
	JMP MCEND

MC13A	LDA SOUNDNO	; DECREMENT SOUND # USING SPINNER
	BEQ MC13C
	DEC SOUNDNO
	LDA #1
	STA SPINFLAG
	JSR SOUND2SID
	JSR UPDATESID
	JSR UPDATESCR
MC13C	JMP MCEND

MC13B	LDA SOUNDNO	; INCREMENT SOUND # USING SPINNER
	CMP #9
	BCS MC13D
	INC SOUNDNO
	LDA #1
	STA SPINFLAG
	JSR SOUND2SID
	JSR UPDATESCR
MC13D	JMP MCEND

MC14	JMP MCEND

MCEND	RTS

;--------------------------------------------------------------------------------

; *** USER CLICKED SOMEWHERE ON THE VOICE SCREEN ***

VOICECLICK = *

	LDX VOFFSET		; ESTABLISH REGISTER OFFSET FOR SELECTED VOICE

	LDA ROW
	CMP #3
	BNE VC4
	LDA COL
	CMP #14
	BNE VC2

	LDA SIDREGS + 1, X	; DECREASE VOICE FREQUENCY (HIGH BYTE)
	CMP #1
	BNE VC1
	LDA #0
	STA SPINDELAY
VC1	DEC SIDREGS + 1, X
VC3A	JSR SHOWFREQHI
	JSR SHOWSLIDEH
	LDA #1
	STA SPINFLAG
	JMP VCEND

VC2	CMP #18
	BNE VC4

	LDA SIDREGS + 1, X	; INCREASE VOICE FREQUENCY (HIGH BYTE)
	CMP #254
	BNE VC3
	LDA #0
	STA SPINDELAY
VC3	INC SIDREGS + 1, X
	JMP VC3A

VC4	LDA ROW
	CMP #4
	BNE VC8
	LDA COL
	CMP #14
	BNE VC6

	LDA SIDREGS, X		; DECREASE VOICE FREQUENCY (LOW BYTE)
	CMP #1
	BNE VC5
	LDA #0
	STA SPINDELAY
VC5	DEC SIDREGS, X
VC7A	JSR SHOWFREQLO
	JSR SHOWSLIDEL
	LDA #1
	STA SPINFLAG
	JMP VCEND

VC6	CMP #18
	BNE VC8

	LDA SIDREGS, X		; INCREASE VOICE FREQUENCY (LOW BYTE)
	CMP #254
	BNE VC7
	LDA #0
	STA SPINDELAY
VC7	INC SIDREGS, X
	JMP VC7A

VC8	CMP #14
	BNE VC9
	LDA COL
	CMP #20
	BCC VC9
	CMP #37
	BCS VC9

	LDA #%00000010		; TOGGLE SYNCHRONIZATION ON/OFF
	EOR SIDREGS + 4, X
	STA SIDREGS + 4, X
	JSR SHOWSYNCBOX
	JMP VCEND

VC9	CMP #16
	BNE VC10
	LDA COL
	CMP #20
	BCC VC10
	CMP #37
	BCS VC10

	LDA #%00000100		; TOGGLE RING MODULATION ON/OFF
	EOR SIDREGS + 4, X
	STA SIDREGS + 4, X
	JSR SHOWRINGBOX
	JMP VCEND

VC10	LDA COL
	CMP #9
	BCC VC14
	CMP #17
	BCS VC14
	LDA ROW

	CMP #16
	BNE VC11

	LDA SIDREGS + 4, X	; TOGGLE TRIANGLE WAVEFORM ON/OFF
	AND #%00011111
	EOR #%00010000
	STA SIDREGS + 4, X
	JMP WAVEBOXES
	JMP VCEND

VC11	CMP #18
	BNE VC12

	LDA SIDREGS + 4, X	; TOGGLE SAWTOOTH WAVEFORM ON/OFF
	AND #%00101111
	EOR #%00100000
	STA SIDREGS + 4, X
	JMP WAVEBOXES
	JMP VCEND

VC12	CMP #20
	BNE VC13

	LDA SIDREGS + 4, X	; TOGGLE PULSE WAVEFORM ON/OFF
	AND #%01001111
	EOR #%01000000
	STA SIDREGS + 4, X
	JMP WAVEBOXES
	JMP VCEND

VC13	CMP #22
	BNE VC14

	LDA SIDREGS + 4, X	; TOGGLE NOISE WAVEFORM ON/OFF
	AND #%10001111
	EOR #%10000000
	STA SIDREGS + 4, X
	JMP WAVEBOXES
	JMP VCEND

WAVEBOXES = *

	JSR SHOWTRIBOX
	JSR SHOWSAWBOX
	JSR SHOWPULBOX
	JSR SHOWNOISBOX
	JMP VCEND

VC14	LDA COL
	CMP #20
	BCC VC14A
	CMP #36
	BCS VC14A
	LDA ROW
	CMP #8
	BNE VC15

	LDA COL			; SET ATTACK VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA TEMP
	LDA SIDREGS + 5, X
	AND #%00001111
	ORA TEMP
	STA SIDREGS + 5, X
	JSR SHOWATTACK
	LDA #1
	STA ALLOWSLIDE
VC14A	JMP VCEND

VC15	CMP #9
	BNE VC16

	LDA COL			; SET DECAY VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	STA TEMP
	LDA SIDREGS + 5, X
	AND #%11110000
	ORA TEMP
	STA SIDREGS + 5, X
	JSR SHOWDECAY
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC16	CMP #10
	BNE VC17

	LDA COL			; SET SUSTAIN VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA TEMP
	LDA SIDREGS + 6, X
	AND #%00001111
	ORA TEMP
	STA SIDREGS + 6, X
	JSR SHOWSUST
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC17	CMP #11
	BNE VC18

	LDA COL			; SET RELEASE VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	STA TEMP
	LDA SIDREGS + 6, X
	AND #%11110000
	ORA TEMP
	STA SIDREGS + 6, X
	JSR SHOWREL
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC18	CMP #22
	BNE VC19

	LDA COL			; SET PULSE WIDTH VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	STA SIDREGS + 3, X
	JSR SHOWPULSE
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC19	CMP #3
	BNE VC20

	LDA COL			; SET VOICE FREQUENCY (HIGH BYTE) BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA SIDREGS + 1, X
	JSR SHOWSLIDEH
	JSR SHOWFREQHI
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC20	CMP #4
	BNE VC21

	LDA COL			; SET VOICE FREQUENCY (LOW BYTE) BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA SIDREGS, X
	JSR SHOWSLIDEL
	JSR SHOWFREQLO
	LDA #1
	STA ALLOWSLIDE
	JMP VCEND

VC21=*


VCEND	RTS

;--------------------------------------------------------------------------------

; *** USER CLICKED SOMEWHERE ON THE FILTER SCREEN ***

FILTERCLICK = *

	LDA ROW
	CMP #10
	BNE FC1
	LDA COL
	CMP #9
	BCC FC1
	CMP #16
	BCS FC1

	LDA #%01000000	; TOGGLE HIGH-PASS FILTER ON/OFF
	EOR SIDREGS + 24
	STA SIDREGS + 24
	JSR SHOWFILTERH
	JMP FCEND

FC1	LDA ROW
	CMP #12
	BNE FC2
	LDA COL
	CMP #9
	BCC FC2
	CMP #16
	BCS FC2

	LDA #%00100000	; TOGGLE BAND-PASS FILTER ON/OFF
	EOR SIDREGS + 24
	STA SIDREGS + 24
	JSR SHOWFILTERB
	JMP FCEND

FC2	LDA ROW
	CMP #14
	BNE FC3
	LDA COL
	CMP #9
	BCC FC3
	CMP #16
	BCS FC3

	LDA #%00010000	; TOGGLE LOW-PASS FILTER ON/OFF
	EOR SIDREGS + 24
	STA SIDREGS + 24
	JSR SHOWFILTERL
	JMP FCEND

FC3	LDA ROW
	CMP #10
	BNE FC4
	LDA COL
	CMP #19
	BCC FC4
	CMP #22
	BCS FC4

	LDA #%00000001	; TOGGLE VOICE 1 FILTER ON/OFF
	EOR SIDREGS + 23
	STA SIDREGS + 23
	JSR SHOWV1FILT
	JMP FCEND

FC4	LDA ROW
	CMP #10
	BNE FC5
	LDA COL
	CMP #24
	BCC FC5
	CMP #27
	BCS FC5

	LDA #%00000010	; TOGGLE VOICE 2 FILTER ON/OFF
	EOR SIDREGS + 23
	STA SIDREGS + 23
	JSR SHOWV2FILT
	JMP FCEND

FC5	LDA ROW
	CMP #10
	BNE FC6
	LDA COL
	CMP #29
	BCC FC6
	CMP #32
	BCS FC6

	LDA #%00000100	; TOGGLE VOICE 3 FILTER ON/OFF
	EOR SIDREGS + 23
	STA SIDREGS + 23
	JSR SHOWV3FILT
	JMP FCEND

FC6	LDA ROW
	CMP #10
	BNE FC7
	LDA COL
	CMP #34
	BCC FC7
	CMP #37
	BCS FC7

	LDA #%00001000	; TOGGLE EXTERNAL VOICE FILTER ON/OFF
	EOR SIDREGS + 23
	STA SIDREGS + 23
	JSR SHOWEFILT
	JMP FCEND

FC7	LDA ROW
	CMP #22
	BNE FC8
	LDA COL
	CMP #12
	BCC FC8
	CMP #29
	BCS FC8

	LDA #%10000000	; TOGGLE VOICE 3 OUTPUT ON/OFF
	EOR SIDREGS + 24
	STA SIDREGS + 24
	JSR SHOWV3OUT
	JMP FCEND

FC8	LDA ROW
	CMP #4
	BNE FC8A
	LDA COL
	CMP #20
	BCC FC8A
	CMP #36
	BCS FC8A

	LDA COL		; SET FILTER CUT-OFF FREQUENCY (UPPER PORTION)
			; BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA TEMP
	LDA SIDREGS + 22
	AND #%00001111
	ORA TEMP
	STA SIDREGS + 22
	JSR FILTERHZHI
	JSR SHOWSLIDEFH
	LDA #1
	STA ALLOWSLIDE
	JMP FCEND

FC8A	LDA ROW
	CMP #5
	BNE FC9
	LDA COL
	CMP #20
	BCC FC9
	CMP #36
	BCS FC9

	LDA COL		; SET FILTER CUT-OFF FREQUENCY (LOWER PORTION)
			; BASED ON SLIDER POSITION
	SEC
	SBC #20
	LSR
	STA SIDREGS + 21
	JSR FILTERHZLO
	JSR SHOWSLIDEFL
	LDA #1
	STA ALLOWSLIDE
	JMP FCEND

FC9	LDA ROW
	CMP #14
	BNE FC10
	LDA COL
	CMP #20
	BCC FC10
	CMP #36
	BCS FC10

	LDA COL		; SET RESONANCE VALUE BASED ON SLIDER POSITION
	SEC
	SBC #20
	ASL
	ASL
	ASL
	ASL
	STA TEMP
	LDA SIDREGS + 23
	AND #%00001111
	ORA TEMP
	STA SIDREGS + 23
	JSR SHOWRESON
	LDA #1
	STA ALLOWSLIDE
	JMP FCEND

FC10	LDA ROW
	CMP #19
	BNE FC11
	LDA COL
	CMP #13
	BCC FC11
	CMP #29
	BCS FC11

	LDA COL		; SET MASTER VOLUME VALUE BASED ON SLIDER POSITION
	SEC
	SBC #13
	STA TEMP
	LDA SIDREGS + 24
	AND #%11110000
	ORA TEMP
	STA SIDREGS + 24
	JSR SHOWMAST
	LDA #1
	STA ALLOWSLIDE
	JMP FCEND

FC11	LDA ROW
	CMP #4
	BNE FC15
	LDA COL
	CMP #14
	BNE FC13

	LDA SIDREGS + 22; DECREASE FILTER CUT-OFF FREQUENCY (HIGH PORTION)
	CMP #1
	BNE FC12
	LDA #0
	STA SPINDELAY
FC12	DEC SIDREGS + 22
FC14A	JSR FILTERHZHI
	JSR SHOWSLIDEFH
	LDA #1
	STA SPINFLAG
	JMP FCEND

FC13	CMP #18
	BNE FC15

	LDA SIDREGS + 22; INCREASE FILTER CUT-OFF FREQUENCY (HIGH PORTION)
	CMP #254
	BNE FC14
	LDA #0
	STA SPINDELAY
FC14	INC SIDREGS + 22
	JMP FC14A

FC15	LDA ROW
	CMP #5
	BNE FC19
	LDA COL
	CMP #14
	BNE FC17

	LDA SIDREGS + 21; DECREASE FILTER CUT-OFF FREQUENCY (LOW PORTION)
	CMP #1
	BNE FC16
	LDA #0
	STA SPINDELAY
FC16	LDX SIDREGS + 21
	DEX
FC18A	TXA
	AND #%00000111
	STA SIDREGS + 21
	JSR FILTERHZLO
	JSR SHOWSLIDEFL
	LDA #1
	STA SPINFLAG
	JMP FCEND

FC17	CMP #18
	BNE FC19

	LDA SIDREGS + 21; INCREASE FILTER CUT-OFF FREQUENCY (LOW PORTION)
	CMP #7
	BNE FC18
	LDA #0
	STA SPINDELAY
FC18	LDX SIDREGS + 21
	INX
	JMP FC18A

FC19=*

FCEND	RTS

;--------------------------------------------------------------------------------

; *** USER CLICKED SOMEWHERE ON THE DONE SCREEN ***

DONECLICK = *

	LDA ROW		; EXIT CLICKED
	CMP #16
	BNE DC1
	LDA COL
	CMP #9
	BCC DC1
	CMP #14
	BCS DC1
	LDA #1
	STA EXITFLAG
	JMP DCEND

DC1	LDA ROW		; SOUND / ALL SOUNDS CLICKED
	CMP #12
	BNE DC6
	LDA COL
	CMP #17
	BCC DC2
	CMP #24
	BCS DC2
	LDA #0
	STA SOUNDFLAG	; SOUND CLICKED
	JMP DCUPDATE
DC2	CMP #26
	BCC DC6
	CMP #31
	BCS DC6
	LDA #1
	STA SOUNDFLAG	; ALL SOUNDS CLICKED
	JMP DCUPDATE

DC6	LDA ROW		; CLEAR CLICKED
	CMP #12
	BNE DC7
	LDA COL
	CMP #9
	BCC DC7
	CMP #14
	BCS DC7
	LDA SOUNDFLAG
	BEQ DC6A
	JSR CLEARALL
	JMP DC6B
DC6A	JSR CLEARSOUND
	JSR INITSID	; INITIALIZE SID REGISTERS
	JSR UPDATESID	; UPDATE SID REGISTERS TO MATCH OUR SHADOW REGISTERS
DC6B	JSR UPDATESCR
	JMP DCEND

DC7	LDA ROW		; LOAD CLICKED
	CMP #8
	BNE DC9
	LDA COL
	CMP #9
	BCC DC8
	CMP #14
	BCS DC8
	LDA FILELEN
	BNE DC7A
	JMP SETFILENAME
DC7A	JSR LOADSOUNDS
	JMP DCEND

DC8	CMP #18		; SAVE CLICKED
	BCC DC9
	CMP #23
	BCS DC9
	LDA FILELEN
	BNE DC8A
	JMP SETFILENAME
DC8A	JSR SAVESOUNDS
	JMP DCEND

DC9	LDA ROW		; FILENAME CLICKED
	CMP #4
	BNE DC10
	LDA COL
	CMP #9
	BCC DC10
	CMP #36
	BCS DC10
	JSR SETFILENAME
	JMP DCEND

DC10=*
DCUPDATE JSR SHOWDCONFIG
DCEND	RTS

FILEPARAMS = *
	LDA #$0
	JSR $FE18	; DISABLE ERROR MESSAGES (FOR DEBUGGING)
	LDA #0		; SET LOGICAL FILE NUMBER
	LDX #8		; SET DEVICE NUMBER
	LDY #1		; SET SECONDARY ADDRESS
	JSR $FE00	; SET LOGICAL FILE NUMBER, DEVICE NUMBER, AND SECONDARY ADDRESS
	LDA FILELEN	; CHARACTER COUNT OF OUR KEYBOARD INPUT BUFFER
	LDX #<FILENAME	; LOW BYTE ADDRESS OF FILENAME BUFFER
	LDY #>FILENAME	; HIGH BYTE ADDRESS OF FILENAME BUFFER
	JSR $FDF9	; SET FILENAME PARAMETERS
	RTS

LOADSOUNDS = *
	JSR FILEPARAMS
	LDA #0		; 0 = LOAD (1 = VERIFY, NOT USED HERE)
	JSR $F49E	; LOAD RAM FROM A DEVICE
	LDA #0		; SET LOGICAL FILE NUMBER
	JSR $F291	; CLOSE A LOGICAL I/O FILE
	JSR SOUND2SID
	JSR UPDATESCR
	RTS

SAVESOUNDS = *
	JSR FILEPARAMS
	LDA #<SOUNDTABLE; LOW BYTE SAVE LOCATION START (SID SHADOW REGISTERS)
	STA $FC
	LDA #>SOUNDTABLE; HIGH BYTE SAVE LOCATION START (SID SHADOW REGISTERS)
	STA $FD
	LDA #$FC	; POINTER TO MEMORY LOCATION TO BE SAVED
	LDX #<SOUNDTEND	; LOW BYTE ENDING ADDRESS FOR SAVE
	LDY #>SOUNDTEND	; HIGH BYTE ENDING ADDRESS FOR SAVE
	JSR $F5DD	; SAVE RAM TO A DEVICE
	LDA #0		; SET LOGICAL FILE NUMBER
	JSR $F291	; CLOSE A LOGICAL I/O FILE
	RTS

;--------------------------------------------------------------------------------

; *** SET OR DISPLAY FILENAME ***

SETFILENAME = *

	JSR SHOWFILENAME

SFN7	LDA BUTTON	; CHECK LEFT MOUSE BUTTON
	AND #%00010000  ; WAIT UNTIL MOUSE BUTTON IS RELEASED
	BEQ SFN7
	LDA #0
	STA KEYCOUNT
	LDA #%00000111
	STA 53269	; RESTORE NORMAL MOUSE POINTER WHILE GETTING INPUT

SFN1	LDA KEYCOUNT	; KEYBOARD INPUT LOOP
	STA BLNSW	; FLASH CURSOR UNLESS THERE IS A KEYPRESS
	BEQ SFN1
	SEI
	LDA #0
	STA BLON	; MAKE SURE THERE ARE NO REMNANTS OF THE CURSOR
	JSR $E5B4	; GET CHARACTER FROM KEYBOARD BUFFER
	CMP #QUOTE
	BEQ SFN1
	CMP #BACKSPACE
	BNE SFN2
	LDX FILELEN	; REMOVE LAST CHARACTER, IF ANY (BACKSPACE PRESSED)
	BEQ SFN1
	DEC FILELEN
	JSR $E716	; OUTPUT TO SCREEN, WITH CURSOR HANDLING
	LDA #SPACE
	STA 1024+195
	LDA #66
	STA 1024+196
	JMP SFN1
SFN2	CMP #RETURN
	BEQ SFN3	; DONE ENTERING FILENAME
	TAX
	AND #$7F	; MASK-OUT REVERSED CHARACTERS
	CMP #SPACE	; MASK-OUT CONTROL CHARACTERS (BELOW SPACE CHAR)
	BCC SFN1
	TXA
	LDX FILELEN	; CHARACTER COUNT OF OUR KEYBOARD INPUT BUFFER
	CPX #16		; MAX FILE LENGTH IS 16 CHARACTERS
	BCS SFN1
	STA FILENAME, X	; STORE CHARACTER WITHIN OUR KEYBOARD INPUT BUFFER
	INC FILELEN
	JSR $E716	; PRINT CHARACTER, IF PRINTABLE
	JMP SFN1
SFN3	JSR SHOWFILENAME
	RTS

SHOWFILENAME = *

	LDA #4		; POSITION CURSOR
	STA CURSORY
	LDA #18
	STA CURSORX
	JSR CURSORPOS

	LDX #17		; CLEAR TEXT FIELD ON SCREEN
	LDA #SPACE
SFN6	JSR CHROUT
	DEX
	BNE SFN6
	JSR CURSORPOS

	LDY FILELEN
	BEQ SFN9
	LDX #0
SFN8	LDA FILENAME, X
	JSR CHROUT
	INX
	DEY
	BNE SFN8

SFN9	LDA #18
	CLC
	ADC FILELEN
	STA CURSORX
	JSR CURSORPOS
	RTS

;--------------------------------------------------------------------------------

; *** UPDATE SCREEN TO REFLECT STATUS OF VARIABLES ***

; THIS FUNCTION CALLS VOICESCREEN, FILTERSCREEN OR DONESCREEN
; DO NOT CALL THOSE FUNCTIONS DIRECTLY

UPDATESCR = *

	PHA		; SELECT SCREEN TO REFRESH, BASED ON MENU #
	TXA
	PHA
	TYA
	PHA
	LDA #255	; FORCE SLIDERS TO BE RE-DRAWN
	STA LASTSLIDEH
	STA LASTSLIDEL
	STA LASTSLIDEFH

	LDA MENUNO
	CMP #3
	BEQ REF1
	CMP #4
	BEQ REF3
	JSR VOICESCREEN
	JMP REF2
REF3	JSR DONESCREEN
	JMP REF2
REF1	JSR FILTERSCREEN
	JMP REF2
REF2	JSR V1BOX
	JSR V2BOX
	JSR V3BOX
	JSR SHOWSOUNDNO
	PLA
	TAY
	PLA
	TAX
	PLA
	RTS

V1BOX	LDX #4		; SHOW V1 CHECKBOX STATUS
	LDY #4
	LDA SIDREGS + 4
	AND #%00000001
	BNE V1A
	JSR CHECKOFF
	JMP V1B
V1A	JSR CHECKON
V1B	RTS

V2BOX	LDX #6		; SHOW V2 CHECKBOX STATUS
	LDY #4
	LDA SIDREGS + 4 + 7
	AND #%00000001
	BNE V2A
	JSR CHECKOFF
	JMP V2B
V2A	JSR CHECKON
V2B	RTS

V3BOX	LDX #8		; SHOW V3 CHECKBOX STATUS
	LDY #4
	LDA SIDREGS + 4 + 14
	AND #%00000001
	BNE V3A
	JSR CHECKOFF
	JMP V3B
V3A	JSR CHECKON
V3B	RTS

SHOWSOUNDNO = *		; SHOW SOUND # (DISPLAY VALUES 0-9 AS 1-10)

	LDX #16
	STX CURSORY
	LDX #2
	STX CURSORX
	JSR CURSORPOS
	LDA SOUNDNO
	CLC
	SED
	ADC #1
	CLD
	PHA
	ROR
	ROR
	ROR
	ROR
	AND #$0F
	ORA #$30
	JSR CHROUT
	PLA
	AND #$0F
	ORA #$30
	JSR CHROUT
	RTS

;--------------------------------------------------------------------------------

; *** SHOW VOICE SCREEN ***

; THIS FUNCTION IS CALLED INDIRECTLY BY UPDATESCR
; DO NOT CALL THIS FUNCTION DIRECTLY

VOICESCREEN = *

	LDA #<VSCREEN
	STA $FA
	LDA #>VSCREEN
	STA $FB
	JSR SHOWSCR	; SHOW VOICE CONFIGURATION SCREEN

	LDX MENUNO	; DISPLAY SELECTED VOICE # AT TOP OF SCREEN (2 PLACES)
	INX
	TXA
	ORA #48
	STA 1039
	STA 1060

	LDA MENUNO	; HIGHLIGHT VOICE MENU SELECTION
	BNE VS1		; V1 MENU SELECTED
	LDY #'3'	; SYNCH VOICES 1 & 3
	STY 1776
	LDY #0		; V1 REGISTERS START AT 54272
	STY VOFFSET

VS1	CMP #1		; V2 MENU SELECTED
	BNE VS2
	LDY #'1'	; SYNCH VOICES 2 & 1
	STY 1776
	LDY #7		; V2 REGISTERS START AT 54272 + 7
	STY VOFFSET
	LDY #80		; CURSOR DOWN 2 LINES

VS2	CMP #2		; V3 MENU SELECTED
	BNE VS3
	LDY #'2'	; SYNCH VOICES 3 & 2
	STY 1776
	LDY #14		; V3 REGISTERS START AT 54272 + 14
	STY VOFFSET
	LDY #160	; CURSOR DOWN 4 LINES

VS3	LDX #2		; HIGHLIGHT MENU SELECTION (2 CHARS)
VS4	LDA 1185, Y
	ORA #128
	STA 1185, Y
	INY
	DEX
	BNE VS4

	JSR SHOWVCONFIG	; DISPLAY VOICE CONFIGURATION

	RTS

;--------------------------------------------------------------------------------

; *** SHOW FILTER SCREEN ***

; THIS FUNCTION IS CALLED INDIRECTLY BY UPDATESCR
; DO NOT CALL THIS FUNCTION DIRECTLY

FILTERSCREEN = *

	LDA #<FSCREEN
	STA $FA
	LDA #>FSCREEN
	STA $FB
	JSR SHOWSCR	; SHOW FILTER CONFIGURATION AND MASTER VOLUME SCREEN
	JSR SHOWFCONFIG	; DISPLAY FILTER CONFIGURATION

	RTS

;--------------------------------------------------------------------------------

; *** SHOW DONE SCREEN ***

; THIS FUNCTION IS CALLED INDIRECTLY BY UPDATESCR
; DO NOT CALL THIS FUNCTION DIRECTLY

DONESCREEN = *

	LDA #<DSCREEN
	STA $FA
	LDA #>DSCREEN
	STA $FB
	JSR SHOWSCR	; SHOW FILTER CONFIGURATION AND MASTER VOLUME SCREEN
	JSR SHOWDCONFIG	; DISPLAY FILTER CONFIGURATION

	RTS

;--------------------------------------------------------------------------------

; *** SHOW VOICE OR FILTER SCREEN (USING PACKBITS DECOMPRESSION) ***

; TO BE CALLED ONLY BY VOICESCREEN OR FILTERSCREEN FUNCTION

SHOWSCR = *

	LDA #<SCREENRAM
	STA $FC
	LDA #>SCREENRAM
	STA $FD

	LDY #0		; INDIRECT OFFSET (NEEDED FOR COPY INSTRUCTION)

SS1	JSR SSSOURCE	; RETRIEVE BYTE OF COMPRESSED DATA
	CMP #0		; 0 = END OF DATA
	BEQ SSEND
	CMP #255	; 255 = REPEATED CHARACTERS FOLLOW
	BEQ SS2
	JSR SSTARGET	; OUTPUT SINGLE BYTE TO SCREEN
	JMP SS1

SS2	JSR SSSOURCE	; RETRIEVE COUNT OF REPEATED CHARACTERS
	TAX		; COUNTDOWN INDEX
	JSR SSSOURCE	; RETRIEVE THE CHARACTER TO BE REPEATED
SS3	JSR SSTARGET	; OUTPUT SINGLE BYTE TO SCREEN
	DEX
	BNE SS3		; REPEAT THE SPECIFIED NUMBER OF TIMES
	JMP SS1

SSEND	RTS

SSSOURCE = *		; RETRIEVE BYTE OF COMPRESSED DATA

	LDA ($FA), Y
	INC $FA
	BNE SSS1
	INC $FB
SSS1	RTS

SSTARGET = *		; OUTPUT SINGLE BYTE OF UNCOMPRESSED DATA TO SCREEN

	STA ($FC), Y
	INC $FC
	BNE SST1
	INC $FD
SST1	RTS

;--------------------------------------------------------------------------------

; *** DISPLAY VOICE CONFIGURATION ***

SHOWVCONFIG = *

	JSR SHOWFREQHI	; SHOW VOICE FREQUENCY (HIGH BYTE)

	JSR SHOWSLIDEH	; SHOW VOICE FREQUENCY (HIGH BYTE) SLIDER VALUE

	JSR SHOWFREQLO	; SHOW VOICE FREQUENCY (LOW BYTE)

	JSR SHOWSLIDEL	; SHOW VOICE FREQUENCY (LOW BYTE) SLIDER VALUE

	JSR SHOWATTACK 	; SHOW ATTACK VALUE

	JSR SHOWDECAY	; SHOW DECAY VALUE

	JSR SHOWSUST	; SHOW SUSTAIN VALUE

	JSR SHOWREL	; SHOW RELEASE VALUE	

	JSR SHOWSYNCBOX	; SHOW SYNCHRONIZATION CHECKBOX

	JSR SHOWRINGBOX	; SHOW RING MODULATION CHECKBOX

	JSR SHOWPULSE	; SHOW PULSE WIDTH VALUE

	JSR SHOWTRIBOX	; SHOW TRIANGLE WAVEFORM CHECKBOX

	JSR SHOWSAWBOX	; SHOW SAWTOOTH WAVEFORM CHECKBOX

	JSR SHOWPULBOX	; SHOW PULSE WAVEFORM CHECKBOX

	JSR SHOWNOISBOX	; SHOW NOISE WAVEFORM CHECKBOX

	RTS

; *** CONTROLS ARE UPDATED INDIVIDUALLY, TO LESSEN FLICKER FROM REDUNDANT RE-DRAWS ***

SHOWFREQHI = *		; SHOW VOICE FREQUENCY (HIGH BYTE)

	LDX #3
	STX CURSORY
	LDX #15
	STX CURSORX
	JSR CURSORPOS
	LDX VOFFSET
	LDA SIDREGS + 1, X
	JSR SHOWDECVAL
	RTS

SHOWSLIDEH = *		; SHOW VOICE FREQUENCY (HIGH BYTE) SLIDER VALUE

	LDX #3
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 1, X
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	CMP LASTSLIDEH
	BEQ SSH1
	STA LASTSLIDEH
	JSR SHOWSLIDER
SSH1	RTS

SHOWFREQLO = *		; SHOW VOICE FREQUENCY (LOW BYTE)

	LDX #4
	STX CURSORY
	LDX #15
	STX CURSORX
	JSR CURSORPOS
	LDX VOFFSET
	LDA SIDREGS, X
	JSR SHOWDECVAL
	RTS

SHOWSLIDEL = *		; SHOW VOICE FREQUENCY (LOW BYTE) SLIDER VALUE

	LDX #4
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS, X
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	CMP LASTSLIDEL
	BEQ SSL1
	STA LASTSLIDEL
	JSR SHOWSLIDER
SSL1	RTS

SHOWSYNCBOX = *		; SHOW SYNCHRONIZATION CHECKBOX STATE

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #14
	LDY #20
	AND #%00000010
	BNE SSB1
	JSR CHECKOFF
	JMP SSB2
SSB1	JSR CHECKON
SSB2	RTS

SHOWRINGBOX = *		; SHOW RING MODULATION CHECKBOX STATE

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #16
	LDY #20
	AND #%00000100
	BNE SRB1
	JSR CHECKOFF
	JMP SRB2
SRB1	JSR CHECKON
SRB2	RTS

SHOWTRIBOX = *		; SHOW TRIANGLE WAVEFORM CHECKBOX STATE

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #16
	LDY #9
	AND #%00010000
	BNE STB1
	JSR CHECKOFF
	JMP STB2
STB1	JSR CHECKON
STB2	RTS

SHOWSAWBOX = *		; SHOW SAWTOOTH WAVEFORM CHECKBOX STATE

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #18
	LDY #9
	AND #%00100000
	BNE SWB1
	JSR CHECKOFF
	JMP SWB2
SWB1	JSR CHECKON
SWB2	RTS

SHOWPULBOX = *		; SHOW PULSE WAVEFORM CHECKBOX STATE

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #20
	LDY #9
	AND #%01000000
	BNE SPB1
	JSR CHECKOFF
	JMP SPB2
SPB1	JSR CHECKON
SPB2	RTS

SHOWNOISBOX = *		; SHOW NOISE WAVEFORM CHECKBOX

	LDX VOFFSET
	LDA SIDREGS + 4, X
	LDX #22
	LDY #9
	AND #%10000000
	BNE SNB1
	JSR CHECKOFF
	JMP SNB2
SNB1	JSR CHECKON
SNB2	RTS

SHOWATTACK = *		; SHOW ATTACK VALUE

	LDX #8
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 5, X
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	JSR SHOWSLIDER
	RTS

SHOWDECAY = *		; SHOW DECAY VALUE

	LDX #9
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 5, X
	JSR SHOWSLIDER
	RTS

SHOWSUST = *		; SHOW SUSTAIN VALUE

	LDX #10
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 6, X
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	JSR SHOWSLIDER
	RTS

SHOWREL = *		; SHOW RELEASE VALUE

	LDX #11
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 6, X
	JSR SHOWSLIDER
	RTS

SHOWPULSE = *		; SHOW PULSE WIDTH VALUE

	LDX #22
	STX CURSORY
	LDX #20
	STX CURSORX
	LDX VOFFSET
	LDA SIDREGS + 3, X
	JSR SHOWSLIDER
	RTS

;--------------------------------------------------------------------------------

; *** DISPLAY FILTER CONFIGURATION ***

SHOWFCONFIG = *

	JSR SHOWFILTERH ; SHOW HIGH-PASS FILTER CHECKBOX

	JSR SHOWFILTERB ; SHOW BAND-PASS FILTER CHECKBOX

	JSR SHOWFILTERL ; SHOW LOW-PASS FILTER CHECKBOX

	JSR SHOWV3OUT	; SHOW VOICE 3 OUTPUT CHECKBOX

	JSR SHOWV1FILT 	; SHOW VOICE 1 FILTER CHECKBOX

	JSR SHOWV2FILT 	; SHOW VOICE 2 FILTER CHECKBOX

	JSR SHOWV3FILT 	; SHOW VOICE 3 FILTER CHECKBOX

	JSR SHOWEFILT 	; SHOW EXTERNAL VOICE FILTER CHECKBOX

	JSR SHOWMAST	; SHOW MASTER VOLUME LEVEL

	JSR SHOWRESON	; SHOW RESONANCE LEVEL

	JSR FILTERHZHI	; FILTER CUT-OFF FREQUENCY (HIGH PORTION)

	JSR FILTERHZLO	; FILTER CUT-OFF FREQUENCY (LOW PORTION)

	JSR SHOWSLIDEFH	; FILTER CUT-OFF FREQUENCY (HIGH PORTION) SLIDER VALUE

	JSR SHOWSLIDEFL	; FILTER CUT-OFF FREQUENCY (LOW PORTION) SLIDER VALUE

	RTS

SHOWFILTERH = *		; SHOW HIGH-PASS FILTER CHECKBOX

	LDX #10
	LDY #9
	LDA SIDREGS + 24
	AND #%01000000
	BNE DFC1A
	JSR CHECKOFF
	JMP DFC1B
DFC1A	JSR CHECKON
DFC1B	RTS

SHOWFILTERB = *		; SHOW BAND-PASS FILTER CHECKBOX

	LDX #12
	LDY #9
	LDA SIDREGS + 24
	AND #%00100000
	BNE DFC2A
	JSR CHECKOFF
	JMP DFC2B
DFC2A	JSR CHECKON
DFC2B	RTS

SHOWFILTERL = *		; SHOW LOW-PASS FILTER CHECKBOX

	LDX #14
	LDY #9
	LDA SIDREGS + 24
	AND #%00010000
	BNE DFC3A
	JSR CHECKOFF
	JMP DFC3B
DFC3A	JSR CHECKON
DFC3B	RTS

SHOWV3OUT = *		; SHOW VOICE 3 OUTPUT CHECKBOX

	LDX #22
	LDY #12
	LDA SIDREGS + 24
	AND #%10000000
	BNE DFC4A
	JSR CHECKOFF
	JMP DFC4B
DFC4A	JSR CHECKON
DFC4B	RTS

SHOWV1FILT = *		; SHOW VOICE 1 FILTER CHECKBOX

	LDX #10
	LDY #19
	LDA SIDREGS + 23
	AND #%00000001
	BNE DFC5A
	JSR CHECKOFF
	JMP DFC5B
DFC5A	JSR CHECKON
DFC5B	RTS

SHOWV2FILT = *		; SHOW VOICE 2 FILTER CHECKBOX

	LDX #10
	LDY #24
	LDA SIDREGS + 23
	AND #%00000010
	BNE DFC6A
	JSR CHECKOFF
	JMP DFC6B
DFC6A	JSR CHECKON
DFC6B	RTS

SHOWV3FILT = *		; SHOW VOICE 3 FILTER CHECKBOX

	LDX #10
	LDY #29
	LDA SIDREGS + 23
	AND #%00000100
	BNE DFC7A
	JSR CHECKOFF
	JMP DFC7B
DFC7A	JSR CHECKON
DFC7B	RTS

SHOWEFILT = *		; SHOW EXTERNAL VOICE FILTER CHECKBOX

	LDX #10
	LDY #34
	LDA SIDREGS + 23
	AND #%00001000
	BNE DFC8A
	JSR CHECKOFF
	JMP DFC8B
DFC8A	JSR CHECKON
DFC8B	RTS

SHOWMAST = *		; SHOW MASTER VOLUME LEVEL

	LDA #19
	STA CURSORY
	LDA #13
	STA CURSORX
	LDA SIDREGS + 24
	JSR SHOWSLIDER
	RTS

SHOWRESON = * 		; SHOW RESONANCE LEVEL

	LDA #14		; RESONANCE
	STA CURSORY
	LDA #20
	STA CURSORX
	LDA SIDREGS + 23
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	JSR SHOWSLIDER
	RTS

SHOWSLIDEFH = *		; FILTER CUT-OFF FREQUENCY (HIGH PORTION) SLIDER VALUE

	LDA #4
	STA CURSORY
	LDA #20
	STA CURSORX
	LDA SIDREGS + 22
	LSR		; WE ONLY CARE ABOUT UPPER NYBBLE
	LSR
	LSR
	LSR
	CMP LASTSLIDEFH
	BEQ SSFH1
	STA LASTSLIDEFH
	JSR SHOWSLIDER
SSFH1	RTS

SHOWSLIDEFL = *		; FILTER CUT-OFF FREQUENCY (LOW PORTION) SLIDER VALUE

	LDA #5
	STA CURSORY
	LDA #20
	STA CURSORX
	LDA SIDREGS + 21
	ASL
	JSR SHOWSLIDER
	RTS

FILTERHZHI = *		; FILTER CUT-OFF FREQUENCY (HIGH PORTION)

	LDX #4
	STX CURSORY
	LDX #15
	STX CURSORX
	JSR CURSORPOS
	LDX VOFFSET
	LDA SIDREGS + 22
	JSR SHOWDECVAL
	RTS

FILTERHZLO = *		; FILTER CUT-OFF FREQUENCY (LOW PORTION)

	LDX #5
	STX CURSORY
	LDX #15
	STX CURSORX
	JSR CURSORPOS
	LDX VOFFSET
	LDA SIDREGS + 21
	JSR SHOWDECVAL
	RTS

;--------------------------------------------------------------------------------

; *** DISPLAY DONE CONFIGURATION ***

SHOWDCONFIG = *

	LDX #12		; SHOW SOUND / ALL RADIO BUTTON SELECTION
	STX CURSORY
	LDX #17
	STX CURSORX
	JSR CURSORPOS
	LDA #RADIOOFF
	JSR CHROUT
	LDA SOUNDFLAG
	BNE SDC1
	JSR CURSORPOS
	LDA #RADIOON
	JSR CHROUT
SDC1	LDX #26
	STX CURSORX
	JSR CURSORPOS
	LDA #RADIOOFF
	JSR CHROUT
	LDA SOUNDFLAG
	BEQ SDC2
	JSR CURSORPOS
	LDA #RADIOON
	JSR CHROUT

SDC2	JSR SHOWFILENAME

	RTS

;--------------------------------------------------------------------------------

; *** DISPLAY CHECKBOX IN ITS CHECKED OR UNCHECKED STATE ***

CHECKON	LDA #CHECKED
	JMP CK1
CHECKOFF LDA #UNCHECKED
CK1	CLC
	PHA
	JSR PLOT
	PLA
	JSR CHROUT
	RTS

;--------------------------------------------------------------------------------

; *** POSITION CURSOR ***

CURSORPOS = *

	PHA
	TXA
	PHA
	TYA
	PHA

	LDX CURSORY
	LDY CURSORX
	CLC
	JSR PLOT

	PLA
	TAY
	PLA
	TAX
	PLA
	RTS

;--------------------------------------------------------------------------------

; *** DISPLAY STRING ***

; SET X REGISTER TO THE OFFSET FROM 'MSG' BEFORE CALLING
; FORMAT IS <ROW>, <COLUMN>, <STRING>, 0

DISPSTR = *

	LDA MSG, X
	STA CURSORY
	INX
	LDA MSG, X
	STA CURSORX
	INX
	JSR CURSORPOS
DS1	LDA MSG, X
	BEQ DS2
	JSR CHROUT
	INX
	BNE DS1
DS2	RTS

;--------------------------------------------------------------------------------

; *** SHOW MOUSE POSITION ***

SHOWMOUSPOS = *

	LDA #23
	STA CURSORY
	LDA #0
	STA CURSORX
	JSR CURSORPOS

	LDA #REVERSEON
	JSR CHROUT
	LDA ROW
	JSR SHOWDECVAL
	LDA #REVERSEOFF
	JSR CHROUT
	INC CURSORY
	JSR CURSORPOS
	LDA #REVERSEON
	JSR CHROUT
	LDA COL
	JSR SHOWDECVAL
	LDA #REVERSEOFF
	JSR CHROUT
	RTS

;--------------------------------------------------------------------------------

; *** SHOW DECIMAL VALUE OF BYTE (LOAD ACCUMULATOR WITH BYTE BEFORE CALLING) ***

SHOWDECVAL = *

	PHA		; CLEAR DIGITS THAT WILL BE USED FOR OUTPUT
	LDA #'0'
	STA DECVAL
	STA DECVAL+1
	PLA

P1	CMP #100
	BCC P2
	SEC
	SBC #100	; HUNDREDS
	INC DECVAL
	JMP P1

P2	CMP #10
	BCC P3
	SEC
	SBC #10		; TENS
	INC DECVAL+1
	JMP P2

P3	ORA #'0'	; ONES
	PHA

	LDA DECVAL
	JSR CHROUT
	LDA DECVAL+1
	JSR CHROUT
	PLA
	JSR CHROUT
	RTS

;--------------------------------------------------------------------------------

; *** SHOW VALUE OF ACCUMULATOR IN A 16-SEGMENT SLIDER ***

SHOWSLIDER = *

	STX TEMP
	PHA
	PHA

	JSR CURSORPOS
	LDX #16
	LDA #'-'
SL1	JSR CHROUT
	DEX
	BNE SL1
	JSR CURSORPOS

	PLA
	AND #%00001111	; WE ONLY CARE ABOUT LOWER NYBBLE
	BEQ SL3
	TAX
	LDA #RIGHT
SL2	JSR CHROUT
	DEX
	BNE SL2
SL3	LDA #SLIDER
	JSR CHROUT

	PLA
	LDX TEMP

	RTS

;--------------------------------------------------------------------------------

; *** SHOW MOUSE POINTER (SPRITES 0-2) ***

SHOWPTR = *

			; 5 * 64 BYTES PER SPRITE = 320 BYTES (256 + 64)
	LDY #0		; RELOCATE POINTER AND OUTLINE SPRITES
SP1	LDA MOUSEPTR, Y
	STA $2800, Y
	INY
	BNE SP1
SP2	LDA MOUSEPTR + $100, Y
	STA $2900, Y
	INY
	CPY #64
	BNE SP2

	LDY #0
	LDA #160	; TARGET LOCATION SPRITE 0 --> $2800 (10240) / 64 = 160
	STA TEMP
SP3	LDA TEMP
	STA 2040, Y
	INC TEMP	; TARGET LOCATION SPRITE 1
	INY
	CPY #8		; SET MEMORY ADDRESSES FOR ALL 8 SPRITES (NOT ALL ARE USED)
	BNE SP3

	LDA #0
	STA 53276	; MULTICOLOR MODE OFF
	STA 53264	; X-POS MSB FOR ALL SPRITES
	LDA #88
	STA 53248	; HORIZONTAL POS SPRITE 0
	STA 53250	; HORIZONTAL POS SPRITE 1
	STA 53252	; HORIZONTAL POS SPRITE 2
	STA 53254	; HORIZONTAL POS SPRITE 3
	STA 53256	; HORIZONTAL POS SPRITE 4
	LDA #102
	STA 53249	; VERTICAL POS SPRITE 0
	STA 53251	; VERTICAL POS SPRITE 1
	STA 53253	; VERTICAL POS SPRITE 2
	STA 53255	; VERTICAL POS SPRITE 3
	STA 53257	; VERTICAL POS SPRITE 4
	LDA #8		; ORANGE
	STA 53287	; SET SPRITE 0 COLOR
	LDA #7		; YELLOW
	STA 53288	; SET SPRITE 1 COLOR
	LDA #1		; WHITE
	STA 53290	; SET SPRITE 3 COLOR
	LDA #0		; BLACK
	STA 53289	; SET SPRITE 2 COLOR
	STA 53291	; SET SPRITE 4 COLOR
	LDA #%00000111
	STA 53269	; ENABLE SPRITES 0-2 (3D POINTER)
	RTS

;--------------------------------------------------------------------------------

; *** HIDE MOUSE POINTER ***

HIDEPTR LDA #0
	STA 53269	; TURN OFF ALL SPRITES
	RTS

;--------------------------------------------------------------------------------

; *** SET UP CUSTOM CHARACTERS ***

CUSTCHAR = *

	LDA 56334
	AND #254
	STA 56334	; TURN OFF KEYSCAN INTERRUPT TIMER
	LDA 1
	AND #251
	STA 1		; SWITCH IN CHARACTER ROM

	LDX #$08	; COPY CHARACTER SET FROM ROM TO RAM
	LDY #$00
	STY $FA
	STY $FC
	LDA #$D0
	STA $FB
	LDA #$38
	STA $FD
CC1	LDA ($FA),Y
	STA ($FC),Y
	DEY
	BNE CC1
	INC $FB
	INC $FD
	DEX
	BNE CC1

	LDA 1
	ORA #4
	STA 1		; SWITCH IN I/O
	LDA 56334
	ORA #1
	STA 56334       ; RESTART KEYSCAN INTERRUPT TIMER
	LDA 53272
	STA OLDVMCSB
	AND #240
	ORA #14		; STARTING LOCATION 14 * 1024 = 14336
	STA 53272       ; ACTIVATE RAM CHARACTER SET

	LDX #0		; INDEX TO ALTERNATE CHARACTERS

CC2	LDA #0		; TARGET BASE ADDRESS (RAM CHARSET)
	STA $FC
	LDA #$38
	STA $FD

	LDA #0		; GET CBM ASCII CODE FOR REPLACEMENT CHARACTER
	STA TEMP	; AND MULTIPLY IT BY 8
	LDA CUSTCHARS, X
	INX
	CLC
	ASL
	JSR MULT
	ASL
	JSR MULT
	ASL
	JSR MULT

	CLC		; ADJUST TARGET ADDRESS FOR REPLACEMENT CHARACTER
	ADC $FC
	STA $FC
	LDA TEMP
	ADC $FD
	STA $FD

	LDY #0		; 8 BYTES TO COPY PER CHARACTER
CC3	LDA CUSTCHARS, X
	STA ($FC), Y
	INX
	INY
	CPY #8
	BNE CC3

	CPX #90		; REPEAT FOR ALL 10 CHARACTERS
	BNE CC2

	RTS

MULT	PHA		; HIGH BYTE FOR MULTIPLICATION ROUTINE
	LDA TEMP
	ROL
	STA TEMP
	PLA
	RTS

;--------------------------------------------------------------------------------

; *** ADDITIONAL FILES TO INCLUDE DURING COMPILATION ***

!src "SS-GRAFX.ASM"	; SCREEN DATA, SPRITES AND CUSTOM CHARACTERS
!src "SS-MOUSE.ASM"	; MOUSE DRIVER